/*
   Name: Stahlring LED Lampe
   Created: 25-02-2016
   Revision: A1.0
   Author: David Schaffner

   Arduino Programm zur Ansteuerung mittels kapazitiven Taster
   Ein-, Auschalten & Dimmen mittels Taster
   Farbwechsel mit Potentiometer
*/

#include <RGBConverter.h>
#include <EEPROM.h>

//Ausgänge LEDs (PWM Ausgänge)
int red = 3;
int green = 5;
int blue = 6;

//Tastereingang
int tast = 9;
//TasterLeds
int tastBlue = 7;
int tastRed = 4;

//Status Helligkeit
int stat = 0;

//Analoger Eingang für Poti (Farbänderung)
int potiIn = A0;

//Zustände Poti
double lastPotiState = 0;

//Farbenarray
byte rgb[] = {0, 0, 0}; // {R,G,B}
double hsv[] = {0, 0, 0}; // {H,S,V}

//Steigende Flanken
//Lampe On oder Off
boolean start = false;
//Status Taster
int state = LOW;
int lastState = LOW;

//Dimmstatus
int dimStat = 2;
//Dimmprogress Stufe
double dimProg = 0.01;

void setup() {
  Serial.begin(9600); //Für Analog Pin
  pinMode(tast, INPUT);
  pinMode(tastRed, OUTPUT);
  pinMode(tastBlue, OUTPUT);

  //Zustände TasterLeds
  digitalWrite(tastBlue, HIGH);
  digitalWrite(tastRed, LOW);
}

void loop() {
  //Taster LED ansteuern
  if (start == true) {
    digitalWrite(tastRed, HIGH);
    digitalWrite(tastBlue, LOW);
  }
  else {
    digitalWrite(tastRed, LOW);
    digitalWrite(tastBlue, HIGH);
  }
  //Digitaler Eingang (Taster) lesen
  state = digitalRead(tast);
  //Analoger Eingang (Potentiometer) lesen und skalieren
  //auf 0 -1 (H-Werte für HSV to RGB Umwandlung)
  hsv[0] = modifiedMap(analogRead(potiIn), 0, 1023, 0, 1);
  //Potentiometer ist der H-Kanal.

  //Änderung des Poti feststellen
  if ((hsv[0] > lastPotiState + 0.01) || (hsv[0] < lastPotiState - 0.01)) {

    if (hsv[0] <= 0.01) {
      lastPotiState = hsv[0];
      hsv[0] = 0; // --> H-Kanal auf 0° = Rot
      if (start == true) {
        writeLed();
      }
    }
    else if (hsv[0] >= 0.99) {
      hsv[1] = 0; //Sättigung auf 0 --> Weiss
      lastPotiState = hsv[0];
      if (start == true) {
        writeLed();
      }
    }
    else {
      if (hsv[1] != 1.00) {
        hsv[1] = 1; //Sättigung wieder 100% -> Alle Farben
      }
      lastPotiState = hsv[0];
      if (start == true) {
        writeLed();
      }
    }
  }

  //steigende Flanke erkennen & Lampe ausschalten
  if (lastState == LOW && state == HIGH && start == true) { //Nur wenn Lampe ON ist
    delay(600); //Wartezeit
    if (digitalRead(tast) == LOW) { //Zum Ausschalten der Lampe
      start = false;
      //Store the color values in the EEPROM storage to have them after a power loss
      EEPROM.write(0, hsv[0] * 100);
      EEPROM.write(1, hsv[1] * 100);
      EEPROM.write(2, hsv[2] * 100);
      //Lampe ausschalten
      analogWrite(red, 0);
      analogWrite(green, 0);
      analogWrite(blue, 0);
      Serial.println("OFF");
      delay(200);
    }
  }

  //Zum Einschalten der Lampe und zum Dimmen
  if (lastState == HIGH && state == HIGH) {
    if (start == false) { //Lampe einschalten
      delay(200); //Warten um zu überprüfen ob nur eingeschaltet wird
      start = true;
      //Restore the colors after power loss or lamp off
      hsv[0] = EEPROM.read(0) * 0.01;
      hsv[1] = EEPROM.read(1) * 0.01;
      hsv[2] = EEPROM.read(2) * 0.01;
      writeLed();
      Serial.println("ON");
    }

    if (digitalRead(tast) != LOW) { //Überprüfung ob nur eingeschaltet wurde
      //Falls "tast" == HIGH, dann wird gedimmt. Sonst wurde nur eingeschaltet.
      Serial.println("Dimmen");
      if (dimStat == 1) {
        stat = 30; //Status wird auf 30 gesetzt: Raufdimmen -> heller
        dimUpDown();
      }
      else {
        stat = 31; //Status wird auf 31 gesetzt: Runterdimmen -> dunkler
        dimUpDown();
      }
    }
  }

  //Zustand merken
  lastState = state;

  //Abfallende Flanke detektieren um Dimmabfolge zu ändern.
  if (lastState == HIGH && digitalRead(tast) == LOW) {
    Serial.println("Dimmstatus Change");
    if (dimStat == 1) {
      dimStat = 2;
    }
    else {
      dimStat = 1;
    }
  }
}

//Methode Auf- und Abdimmen
void dimUpDown() {

  switch (stat) {
    case 30:
      Serial.println("DimUp");
      if (hsv[2] >= 1.00) {
        hsv[2] = 0.99;
      }
      hsv[2] = hsv[2] + dimProg;
      writeLed();

      if (hsv[2] >= 0.98) { //Nicht auf 1, da sonst Code irgendwie spinnt :)
        dimStat = 2;
        delay(1500);
      }
      break;

    case 31:
      Serial.println("DimDown");
      if (hsv[2] <= 0.00) {
        hsv[2] = 0.01;
      }
      hsv[2] = hsv[2] - dimProg;
      writeLed();

      if (hsv[2] <= 0.01) { //Geht nur bis 0.01, da sonst die if-Schleife spinnt (negative Zahhlen)
        dimStat = 1;
        delay(1500);
      }
      break;
  }
}

/**
   Converts an HSV color value to RGB.
   Assumes h, s, and v are contained in the set [0, 1] and
   returns r, g, and b in the set [0, 255].

   @param   Number  h       The hue
   @param   Number  s       The saturation
   @param   Number  v       The value
   @return  Array           The RGB representation
*/

void hsvToRgb(double h, double s, double v, byte rgb[]) {

  //   Serial.print("H:");
  //   Serial.print(h);
  //   Serial.print(" S:");
  //   Serial.print(s);
  //   Serial.print(" V:");
  //   Serial.println(v);

  double r, g, b;

  int i = int(h * 6);
  double f = h * 6 - i;
  double p = v * (1 - s);
  double q = v * (1 - f * s);
  double t = v * (1 - (1 - f) * s);

  switch (i % 6) {
    case 0:
      r = v, g = t, b = p;
      break;
    case 1:
      r = q, g = v, b = p;
      break;
    case 2:
      r = p, g = v, b = t;
      break;
    case 3:
      r = p, g = q, b = v;
      break;
    case 4:
      r = t, g = p, b = v;
      break;
    case 5:
      r = v, g = p, b = q;
      break;
  }

  if (hsv[2] <= 0.02 && dimStat == 2) { //Ausnahme, da sonst nicht ganz dunkel
    rgb[0] = 0;
    rgb[1] = 0;
    rgb[2] = 0;
  }
  else {
    rgb[0] = r * 255;
    rgb[1] = g * 255;
    rgb[2] = b * 255;
  }
  if (rgb[0] >= 252 && dimStat == 1) {
    rgb[0] = 255;  //Ausnahme, da sonst nicht ganz hell
  }
  if (rgb[1] >= 252 && dimStat == 1) {
    rgb[1] = 255;
  }
  if (rgb[2] >= 252 && dimStat == 1) {
    rgb[2] = 255;
  }
}

double modifiedMap(double x, double in_min, double in_max, double out_min, double out_max) {
  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
}

void writeLed() {
  hsvToRgb(hsv[0], hsv[1], hsv[2], rgb); //Umrechnung HSV to RGB
  analogWrite(red, rgb[0]);
  analogWrite(green, rgb[1]);
  analogWrite(blue, rgb[2]);
  Serial.print("R:");
  Serial.print(rgb[0]);
  Serial.print(" G:");
  Serial.print(rgb[1]);
  Serial.print(" B:");
  Serial.println(rgb[2]);
}
